@@ -23,6 +23,7 @@ import errno
 import getpass
 import http.client
 import json
+import logging
 import os
 import pipes
 import queue
@@ -100,7 +101,7 @@ class MirrorStatus(object):
   )
 
   # Known system architectures, as used to replace the "$arch" variable.
-  ARCHITECTURES = ['i686', 'x86_64']
+  ARCHITECTURES = ['x86_64']
 
   # Initialize
   # refresh_interval:
@@ -115,6 +116,7 @@ class MirrorStatus(object):
     connection_timeout=5,
 #     download_timeout=None,
     cache_timeout=300,
+    min_completion_pct=1.,
     threads=5
   ):
     self.refresh_interval = refresh_interval
@@ -131,23 +133,13 @@ class MirrorStatus(object):
 #     self.download_timeout = download_timeout
     # Cache timeout
     self.cache_timeout = cache_timeout
+    # Minimum completion percent, for filtering mirrors.
+    self.min_completion_pct = min_completion_pct
     # Threads
     self.threads = threads
 
 
 
-  def log(self, msg):
-    '''
-    "Log" messages.
-
-    For now this just prints data to STDERR when in verbose mode.
-    '''
-    if self.verbose:
-#       timestamp = time.strftime('[%Y-%m-%d %H:%M:%S]', time.localtime())
-#       sys.stderr.write(timestamp + ' ' + msg)
-      sys.stderr.write(msg)
-
-
   def retrieve(self):
     """Retrieve the current mirror status JSON data."""
     self.json_obj = None
@@ -187,7 +179,11 @@ class MirrorStatus(object):
       # comparison later.
       mirrors = self.json_obj['urls']
       # Filter incomplete mirrors  and mirrors that haven't synced.
-      mirrors = list(m for m in mirrors if m['last_sync'] and m['completion_pct'] == 1.0)
+      mirrors = list(
+        m for m in mirrors
+        if m['last_sync']
+        and m['completion_pct'] >= self.min_completion_pct
+      )
       # Parse 'last_sync' times for future comparison.
       for mirror in mirrors:
         mirror['last_sync'] = calendar.timegm(
@@ -196,7 +192,7 @@ class MirrorStatus(object):
         )
       self.json_obj['urls'] = mirrors
     except KeyError:
-      raise MirrorStatusError('failed to parse retrieved mirror data (the format may have changed or there may be a transient error)' % (e))
+      raise MirrorStatusError('failed to parse retrieved mirror data (the format may have changed or there may be a transient error)')
 
     if save_json and json_str:
       try:
@@ -248,12 +244,14 @@ class MirrorStatus(object):
     t = time.time()
     n = 0
 
+    # Make country arguments case-insensitive.
+    uc_countries = tuple(c.upper() for c in countries) if countries else None
     for mirror in mirrors:
       # Filter by country.
       if countries \
-      and not (\
-        mirror['country'] in countries or \
-        mirror['country_code'] in countries \
+      and not ( \
+        mirror['country'].upper() in uc_countries or \
+        mirror['country_code'].upper() in uc_countries \
       ):
         continue
       # Filter by protocol.
@@ -331,7 +329,7 @@ class MirrorStatus(object):
     def worker():
       while True:
         url = q_in.get()
-        db_subpath = 'core/os/i686/core.db'
+        db_subpath = 'core/os/x86_64/core.db'
         db_url = url + db_subpath
         scheme = urllib.parse.urlparse(url).scheme
         # Leave the rate as 0 if the connection fails.
@@ -384,10 +382,9 @@ class MirrorStatus(object):
 
 
     # Load the input queue.Queue
-    url_len = 0
+    url_len = max(len(m['url']) for m in mirrors)
     for mirror in mirrors:
-      url_len = max(url_len, len(mirror['url']))
-      self.log("rating {}\n".format(mirror['url']))
+      logging.info("rating {}".format(mirror['url']))
       q_in.put(mirror['url'])
 
     q_in.join()
@@ -398,16 +395,16 @@ class MirrorStatus(object):
     # items is retrieved.
 
     # Display some extra data.
-    url_len = str(url_len)
-    self.log(('%-' + url_len + 's  %14s  %9s\n') % ('Server', 'Rate', 'Time'))
-    fmt = '%-' + url_len + 's  %8.2f KiB/s  %7.2f s\n'
+    header_fmt = '{{:{:d}s}}  {{:>14s}}  {{:>9s}}'.format(url_len)
+    logging.info(header_fmt.format('Server', 'Rate', 'Time'))
+    fmt = '{{:{:d}s}}  {{:8.2f}} KiB/s  {{:7.2f}} s'.format(url_len)
 
     # Loop over the mirrors just to ensure that we get the rate for each mirror.
     # The value in the loop does not (necessarily) correspond to the mirror.
     for mirror in mirrors:
       url, rate, dt = q_out.get()
       kibps = rate / 1024.0
-      self.log(fmt % (url, kibps, dt))
+      logging.info(fmt.format(url, kibps, dt))
       rates[url] = rate
       q_out.task_done()
 
@@ -632,6 +629,11 @@ def add_arguments(parser):
     help='Limit the list to the n most recently synchronized servers.'
   )
 
+  filters.add_argument(
+    '--score', type=int, metavar='n',
+    help='Limit the list to the n servers with the highest score.'
+  )
+
   filters.add_argument(
     '-n', '--number', type=int, metavar='n',
     help='Return at most n mirrors.'
@@ -642,6 +644,11 @@ def add_arguments(parser):
     help='Match one of the given protocols, e.g. "http", "ftp".'
   )
 
+  filters.add_argument(
+    '--completion-percent', type=float, metavar='[0-100]', default=100.,
+    help='Set the minimum completion percent for the returned mirrors. Check the mirrorstatus webpage for the meaning of this parameter. Default value: %(default)s.'
+  )
+
   return parser
 
 
@@ -673,6 +680,7 @@ def process_options(options, ms=None, mirrors=None):
       connection_timeout=options.connection_timeout,
 #       download_timeout=options.download_timeout,
       cache_timeout=options.cache_timeout,
+      min_completion_pct=(options.completion_percent/100.),
       threads=options.threads
     )
 
@@ -693,6 +701,10 @@ def process_options(options, ms=None, mirrors=None):
     mirrors = ms.sort(mirrors, by='age')
     mirrors = mirrors[:options.latest]
 
+  if options.score and options.score > 0:
+    mirrors = ms.sort(mirrors, by='score')
+    mirrors = mirrors[:options.score]
+
   if options.fastest and options.fastest > 0:
     mirrors = ms.sort(mirrors, by='rate')
     mirrors = mirrors[:options.fastest]
@@ -709,7 +721,7 @@ def process_options(options, ms=None, mirrors=None):
 
 
 
-def main(args=None):
+def main(args=None, configure_logging=False):
   if args:
     cmd = tuple(args)
   else:
@@ -717,6 +729,17 @@ def main(args=None):
 
   options = parse_args(args)
 
+  if configure_logging:
+    if options.verbose:
+      level = logging.INFO
+    else:
+      level = logging.WARNING
+    logging.basicConfig(
+      format='[{asctime:s}] {levelname:s}: {message:s}',
+      style='{',
+      datefmt='%Y-%m-%d %H:%M:%S',
+      level=level
+    )
 
   try:
     ms, mirrors = process_options(options)
@@ -747,13 +770,13 @@ def main(args=None):
 
 
 
-def run_main(args=None):
+def run_main(args=None, **kwargs):
   try:
-    main(args)
+    main(args, **kwargs)
   except KeyboardInterrupt:
     pass
 
 
 if __name__ == "__main__":
-  run_main()
+  run_main(configure_logging=True)
 
